import logging
from django.views.generic.detail import SingleObjectMixin
from django.template.loader import render_to_string
from jsonbase import JSONResponseMixin
from pgcontent.utils import get_object_by_ctype_id


log = logging.getLogger("pu.in.core")


class InlineActionMixin(JSONResponseMixin):

    """ Handle action in a JSON way.
    """

    handle_on_get = False
    handle_on_post = True    

    def handle_request(self, *args, **kwargs):

        """ Implement handle call to actually do something... Must
        return a tuple of (status, errors) """

        raise NotImplementedError

    def get(self, request, *args, **kwargs):

        if self.handle_on_get:
            kwargs['status'], kwargs['errors'] = self.handle_request()
        else:
            kwargs['status'] = 0
            kwargs['errors'] = ""

        return self.render_to_response(self.get_context_data(**kwargs));

    def post(self, request, *args, **kwargs):

        if self.handle_on_post:
            kwargs['status'], kwargs['errors'] = self.handle_request()
        else:
            kwargs['status'] = 0
            kwargs['errors'] = ""

        return self.render_to_response(self.get_context_data(**kwargs));


class InlineObjectActionMixin(InlineActionMixin, SingleObjectMixin):

    """ Action on object """

    def get_context_data(self, **kwargs):

        """ Base implementation that just returns the view's kwargs """

        context = super(InlineObjectActionMixin, 
                        self).get_context_data(**kwargs)

        context['object'] = self.object

        return context

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        
        return super(InlineObjectActionMixin, self).get(request, *args, 
                                                        **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        
        return super(InlineObjectActionMixin, self).post(request, *args, 
                                                        **kwargs)


class InlineCTObjectActionMixin(InlineObjectActionMixin):

    def get_object(self):

        return get_object_by_ctype_id(self.kwargs['ctype'], self.kwargs['id'])
    



from django import http
from django.utils import simplejson as json
from django.template.loader import render_to_string


class JSONResponseMixin(object):

    template_name = None

    def render_html(self, context, template=None):

        """ Override this so as to return an actual html
        template. This will be added to the JSON data under the key of
        'html'.
        """

        if not template:
            template = self.template_name

        return template and render_to_string(template, context) or ""

    def get_context_data(self, **kwargs):

        """ Base implementation that just returns the view's kwargs """

        kwargs['request'] = self.request

        return kwargs

    def render_to_response(self, context, template=None, **response_kwargs):
        
        "Returns a JSON response containing 'context' as payload"

        context['html'] = self.render_html(context, template=template)

        return http.HttpResponse(
            json.dumps(context, skipkeys=True,
                       default=lambda x: "NOT SERIALIZABLE"),
            content_type='application/json',
            **response_kwargs)


class JSONFormMixin(JSONResponseMixin):

    success_template_name = None

    def form_valid(self, form):

        self.object = form.save()
        return self.render_to_response(
            self.get_context_data(form=form, object=self.object),
            template=self.success_template_name)

    def get_context_data(self, **kwargs):

        context = super(JSONFormMixin, self).get_context_data(**kwargs)

        context['action'] = self.request.path

        return context
